#pragma once

#include "OfficeArtContainer.h"

namespace ODRAW
{

class OfficeArtDggContainer : public OfficeArtContainer
{
	BASE_STRUCTURE_DEFINE_CLASS_NAME(OfficeArtDggContainer)
public:
	OfficeArtDggContainer(const OfficeArtClientAnchorType anchor_type) : OfficeArtContainer(0x0F, DggContainer, anchor_type) {}
	XLS::BiffStructurePtr clone() { return XLS::BiffStructurePtr(new OfficeArtDggContainer(*this)); }

	void loadFields(XLS::CFRecord& record);

	OfficeArtRecordPtr		m_OfficeArtBStoreContainer;
	OfficeArtRecordPtr		m_OfficeArtColorMRUContainer;
	OfficeArtRecordPtr		m_OfficeArtSplitMenuColorContainer;
	OfficeArtRecordPtr		m_OfficeArtFDGGBlock;
	//+ OfficeArtFOPT + OfficeArtTertiaryFOPT
};

class OfficeArtSpgrContainer : public OfficeArtContainer
{
	BASE_STRUCTURE_DEFINE_CLASS_NAME(OfficeArtSpgrContainer)
public:
	OfficeArtSpgrContainer(const OfficeArtClientAnchorType anchor_type) : OfficeArtContainer(0x0F, SpgrContainer, anchor_type) {}
	XLS::BiffStructurePtr clone() { return XLS::BiffStructurePtr(new OfficeArtSpgrContainer(*this)); }

	void loadFields(XLS::CFRecord& record);

	std::vector<OfficeArtContainerPtr>		m_OfficeArtSpgrContainerFileBlock;
};

class OfficeArtSolverContainer : public OfficeArtRecord
{
	BASE_STRUCTURE_DEFINE_CLASS_NAME(OfficeArtSolverContainer)
public:
	OfficeArtSolverContainer() : OfficeArtRecord(0x0F, SolverContainer) {}
	XLS::BiffStructurePtr clone() { return XLS::BiffStructurePtr(new OfficeArtSolverContainer(*this)); }

	void loadFields(XLS::CFRecord& record);
	void storeFields(XLS::CFRecord& record) {}
};

class OfficeArtSpContainer : public OfficeArtContainer
{
	BASE_STRUCTURE_DEFINE_CLASS_NAME(OfficeArtSpContainer)
public:
	OfficeArtSpContainer(const OfficeArtClientAnchorType anchor_type) : OfficeArtContainer(0x0F, SpContainer, anchor_type) {}
	XLS::BiffStructurePtr clone() { return XLS::BiffStructurePtr(new OfficeArtSpContainer(*this)); }

	void loadFields(XLS::CFRecord& record);

	OfficeArtRecordPtr m_OfficeArtFSP;
	OfficeArtRecordPtr m_OfficeArtAnchor;
};

class OfficeArtClientData : public OfficeArtRecord
{
	BASE_STRUCTURE_DEFINE_CLASS_NAME(OfficeArtClientData)
public:
	OfficeArtClientData() : OfficeArtRecord(0, ClientData) {}
	XLS::BiffStructurePtr clone() { return XLS::BiffStructurePtr(new OfficeArtClientData(*this)); }
	
	void loadFields(XLS::CFRecord& record);
	void storeFields(XLS::CFRecord& record) {}

	static const XLS::ElementType	type = XLS::typeOfficeArtClientData;

};

class OfficeArtFConnectorRule : public OfficeArtRecord
{
	BASE_STRUCTURE_DEFINE_CLASS_NAME(OfficeArtFConnectorRule)
public:
	OfficeArtFConnectorRule() : OfficeArtRecord(0, FConnectorRule) {}
	XLS::BiffStructurePtr clone() { return XLS::BiffStructurePtr(new OfficeArtFConnectorRule(*this)); }
	
	virtual void loadFields(XLS::CFRecord& record);
	virtual void storeFields(XLS::CFRecord& record) {}

	static const XLS::ElementType	type = XLS::typeOfficeArtClientData;//FConnectorRule

	_UINT32 ruid;
	_UINT32 spidA;
	_UINT32 spidB;
	_UINT32 spidC;
	_UINT32 cptiA;
	_UINT32 cptiB;
};

class OfficeArtFCalloutRule : public OfficeArtRecord
{
	BASE_STRUCTURE_DEFINE_CLASS_NAME(OfficeArtFCalloutRule)
public:
	OfficeArtFCalloutRule() : OfficeArtRecord(0, FCalloutRule) {}
	XLS::BiffStructurePtr clone() { return XLS::BiffStructurePtr(new OfficeArtFCalloutRule(*this)); }
	
	virtual void loadFields(XLS::CFRecord& record);
	virtual void storeFields(XLS::CFRecord& record) {}

	static const XLS::ElementType	type = XLS::typeOfficeArtClientData;//FCalloutRule

	_UINT32 ruid;
	_UINT32 spid;
};

class OfficeArtFArcRule : public OfficeArtRecord
{
	BASE_STRUCTURE_DEFINE_CLASS_NAME(OfficeArtFArcRule)
public:
	OfficeArtFArcRule() : OfficeArtRecord(0, FArcRule) {}
	XLS::BiffStructurePtr clone() { return XLS::BiffStructurePtr(new OfficeArtFArcRule(*this)); }
	
	virtual void loadFields(XLS::CFRecord& record);
	virtual void storeFields(XLS::CFRecord& record) {}

	static const XLS::ElementType	type = XLS::typeOfficeArtClientData;//FArcRule

	_UINT32 ruid;
	_UINT32 spid;
};

class OfficeArtClientTextbox : public OfficeArtRecord
{
	BASE_STRUCTURE_DEFINE_CLASS_NAME(OfficeArtClientTextbox)
public:
	OfficeArtClientTextbox() : OfficeArtRecord(0, ClientTextbox) {}
	XLS::BiffStructurePtr clone() { return XLS::BiffStructurePtr(new OfficeArtClientTextbox(*this)); }
	
	void loadFields(XLS::CFRecord& record);
	void storeFields(XLS::CFRecord& record) {}

	static const XLS::ElementType	type = XLS::typeOfficeArtClientTextbox;


};



typedef boost::shared_ptr<OfficeArtSpgrContainer> OfficeArtSpgrContainerPtr;
typedef boost::shared_ptr<OfficeArtSpContainer> OfficeArtSpContainerPtr;
typedef boost::shared_ptr<OfficeArtClientData> OfficeArtClientDataPtr;
typedef boost::shared_ptr<OfficeArtClientTextbox> OfficeArtClientTextboxPtr;


} // namespace XLS
