
#include "CELL.h"
#include <Logic/Biff_records/Blank.h>
#include <Logic/Biff_unions/FORMULA.h>
#include <Logic/Biff_records/MulBlank.h>
#include <Logic/Biff_records/RK.h>
#include <Logic/Biff_records/MulRk.h>
#include <Logic/Biff_records/BoolErr.h>
#include <Logic/Biff_records/Number.h>
#include <Logic/Biff_records/LabelSst.h>
#include <Logic/Biff_records/Label.h> //   SchetPrintForm.xls

namespace XLS
{

CELL::CELL(std::vector<CellRangeRef>& shared_formulas_locations_ref) : shared_formulas_locations_ref_(shared_formulas_locations_ref)
{
	ColumnNumber	= -1;
	RowNumber		= -1;
}


CELL::~CELL()
{
}


BaseObjectPtr CELL::clone()
{
	return BaseObjectPtr(new CELL(*this));
}


// CELL = FORMULA / Blank / MulBlank / RK / MulRk / BoolErr / Number / LabelSst
const bool CELL::loadContent(BinProcessor& proc)
{
	FORMULA formula_union(shared_formulas_locations_ref_);
	
	Blank		blank;
	MulBlank	mulblank;
	RK			rk;
	MulRk		mulrk;
	BoolErr		boolerr;
	Number		number;
	LabelSst	labelsst;
	Label		label;

	if(proc.optional(formula_union))
	{
		RowNumber		= formula_union.getLocation().getRow();
		ColumnNumber	= formula_union.getLocation().getColumn();
	}
	else if(proc.optional(blank))
	{
		RowNumber		= blank.getLocation().getRow();
		ColumnNumber	= blank.getLocation().getColumn();
	}
	else if(proc.optional(mulblank))
	{
		RowNumber		= mulblank.GetRow();
		ColumnNumber	= mulblank.GetColumn();
	}
	else if(proc.optional(rk))
	{
		RowNumber		= rk.getLocation().getRow();
		ColumnNumber	= rk.getLocation().getColumn();
	}
	else if(proc.optional(mulrk))
	{
		RowNumber		= mulrk.GetRow();
		ColumnNumber	= mulrk.GetColumn();
	}
	else if(proc.optional(boolerr))
	{
		RowNumber		= boolerr.getLocation().getRow();
		ColumnNumber	= boolerr.getLocation().getColumn();
	}
	else if(proc.optional(number))
	{
		RowNumber		= number.getLocation().getRow();
		ColumnNumber	= number.getLocation().getColumn();
	}
	else if(proc.optional(label))//
	{
		RowNumber		= label.cell.getLocation().getRow();
		ColumnNumber	= label.cell.getLocation().getColumn();
	}
	else if(proc.optional(labelsst))
	{
		RowNumber		= labelsst.getLocation().getRow();
		ColumnNumber	= labelsst.getLocation().getColumn();
	}
	else
	{
		return false;
	}
	return true;
}

int CELL::serialize(std::wostream & stream)
{
	CP_XML_WRITER(stream)    
    {
		for (std::list<XLS::BaseObjectPtr>::iterator it = elements_.begin(); it != elements_.end(); it++)
		{
			it->get()->serialize(stream);
		}
	}
	return 0;
}

} // namespace XLS

