
#include "IVAXIS.h"
#include <Logic/Biff_records/Axis.h>
#include <Logic/Biff_records/Begin.h>
#include <Logic/Biff_records/CatSerRange.h>
#include <Logic/Biff_records/AxcExt.h>
#include <Logic/Biff_records/CatLab.h>
#include <Logic/Biff_unions/AXS.h>
#include <Logic/Biff_unions/CRTMLFRT.h>
#include <Logic/Biff_records/End.h>

namespace XLS
{


IVAXIS::IVAXIS()
:	id(0), bDataAxis(false)
{
	m_bSecondary = false;
}


IVAXIS::~IVAXIS()
{
}


BaseObjectPtr IVAXIS::clone()
{
	return BaseObjectPtr(new IVAXIS(*this));
}


const bool IVAXIS::loadContentRead(BinReaderProcessor& proc)
{
	bool ok = loadContent(proc);
	if(ok)
	{
		id = proc.getGlobalWorkbookInfo()->GenerateAXESId();
	}
	return ok;
}

// IVAXIS = Axis Begin [CatSerRange] AxcExt [CatLab] AXS [CRTMLFRT] End
const bool IVAXIS::loadContent(BinProcessor& proc)
{
	// This is the way to distinguish IVAXIS and DVAXIS
	if(!proc.checkNextRecord(rt_AxcExt, 4) && !proc.checkNextRecord(rt_CatSerRange, 4))
	{
		return false;
	}
	if(!proc.mandatory<Axis>())
	{
		return false;
	}
	m_Axis = elements_.back();
	elements_.pop_back();

	proc.mandatory<Begin>();					elements_.pop_back();
	
	if (proc.optional<CatSerRange>())
	{
		m_CatSerRange = elements_.back();
		elements_.pop_back();
	}

	if (proc.mandatory<AxcExt>()) // 4-th place
	{
		m_AxcExt = elements_.back();
		elements_.pop_back();
		
		AxcExt *ext = dynamic_cast<AxcExt *>(m_AxcExt.get());
		bDataAxis = ext->fDateAxis;
	}

	if (proc.optional<CatLab>())
	{
		m_CatLab = elements_.back();
		elements_.pop_back();
	}
	if (proc.mandatory<AXS>())
	{
		m_AXS = elements_.back();
		elements_.pop_back();
	}
	if (proc.optional<CRTMLFRT>())
	{
		m_CRTMLFRT = elements_.back();
		elements_.pop_back();
	}
	
	// fix
	proc.optional<CRTMLFRT>(); ///????

	if (proc.mandatory<End>())				elements_.pop_back();

	return true;
}

int IVAXIS::serialize(std::wostream & _stream)
{
	CatSerRange * cat_ser_range = dynamic_cast<CatSerRange*>(m_CatSerRange.get());
	Axis		* axis			= dynamic_cast<Axis*>		(m_Axis.get());
	AxcExt		* axcExt		= dynamic_cast<AxcExt*>		(m_AxcExt.get());

	int axes_type = axis->wType + 1;

	CP_XML_WRITER(_stream)    
	{
		CP_XML_NODE(L"c:axId")
		{
			CP_XML_ATTR(L"val", id);
		}
		
		CP_XML_NODE(L"c:scaling")
		{
			if (cat_ser_range->fReversed)
			{
				CP_XML_NODE(L"c:orientation"){  CP_XML_ATTR(L"val", L"maxMin"); }
			}else
			{
				CP_XML_NODE(L"c:orientation"){  CP_XML_ATTR(L"val", L"minMax"); }
			}
		}
		CP_XML_NODE(L"c:auto")	{  CP_XML_ATTR(L"val", 0); }
		CP_XML_NODE(L"c:delete"){  CP_XML_ATTR(L"val", 0); }
		
//-------------------------------------------------------------------------------
		CP_XML_NODE(L"c:axPos")
		{
			if (axes_type == 1)
			{
				if (m_bSecondary)	CP_XML_ATTR(L"val", L"t");
				else				CP_XML_ATTR(L"val", L"b");
			}
			else if (axes_type == 2)
			{
				if (m_bSecondary)	CP_XML_ATTR(L"val", L"r");
				else				CP_XML_ATTR(L"val", L"l");
			}
			else
			{
				if (m_bSecondary)	CP_XML_ATTR(L"val", L"t");
				else				CP_XML_ATTR(L"val", L"b");
			}
		}
//-----------------------------------------------------------------------------------
		m_AXS->serialize(_stream);

		if (m_AxcExt)
			m_AxcExt->serialize(_stream);
	
		CP_XML_NODE(L"c:crosses")
		{
			if ((cat_ser_range) && (cat_ser_range->fMaxCross == true))	CP_XML_ATTR(L"val", L"max");
			else CP_XML_ATTR(L"val", L"autoZero");
		}	
	}
	
	return 	axes_type;
}
} // namespace XLS

